#pragma once
#include "stdint.h"

// NIH http://wiki.osdev.org/Inline_Assembly/Examples#I.2FO_access
static inline void outb(uint16_t port, uint8_t val)
{
    asm volatile("outb %0, %1" : : "a"(val), "Nd"(port));
    /* There's an outb %al, $imm8  encoding, for compile-time constant port numbers that fit in
     * 8b.  (N constraint). Wider immediate constants would be truncated at assemble-time (e.g.
     * "i" constraint). The  outb  %al, %dx  encoding is the only option for all other cases.
     * %1 expands to %dx because  port  is a uint16_t.  %w1 could be used if we had the port
     * number a wider C type */
}

static inline void outw(uint16_t port, int16_t val)
{
    asm volatile("outw %w0, %w1" :: "a"(val), "Nd"(port));
}

static inline uint8_t inb(uint16_t port)
{
    uint8_t ret;
    asm volatile("inb %1, %0" : "=a"(ret) : "Nd"(port));
    return ret;
}

static inline uint16_t inw(uint16_t port)
{
    uint16_t ret;
    asm volatile("inw %w1, %0" : "=a"(ret) : "Nd"(port));
    return ret;
}
